<?php
/**
 * Class WC_Gateway_HoloPay
 *
 * @category Payment_Gateways
 * @class    WC_Gateway_HoloPay
 * @package  WooCommerce
 * 
 */

class WC_Gateway_HoloPay extends WC_Payment_Gateway
{
	public $webhook_url;
	public $api_key;
	public $paypal_client_id;
	public $stripe_publishable_key;
	public $stripe_secret_key;

	public function __construct()
	{
		global $woocommerce;
		$this->id = 'holopay';
		$this->icon = apply_filters('woocommerce_holopay_icon', HOLOPAY_BASE_URL . '/assets/images/logo.png');
		$this->method_title = __('Holopay', 'holo-pay');
		$this->has_fields = true;
		$this->webhook_url = add_query_arg('wc-api', 'holopay_webhook_handler', home_url('/'));
		$this->init_form_fields();
		$this->init_settings();
		$this->title = $this->get_option('title');
		$this->api_key = $this->get_option('api_key');
		$this->paypal_client_id = $this->get_option('paypal_client_id');
		$this->stripe_publishable_key = $this->get_option('stripe_publishable_key');
		$this->stripe_secret_key = $this->get_option('stripe_secret_key');

		// Actions
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
		// Webhook Handler
		add_action('woocommerce_api_holopay_webhook_handler', [$this, 'webhook_handler']);
	}
	
	public function is_valid_for_use()
	{
		return true;
	}

	/**
	 * Admin Panel Options
	 */
	public function admin_options()
	{
		?>
		<h3><?php esc_html_e('Holopay', 'holo-pay'); ?></h3>

		<table class="form-table">
			<?php
			$this->generate_settings_html();
			?>
		</table>
		<?php
	}
	/**
	 * Initialise settings
	 */
	public function init_form_fields()
	{
		$this->form_fields = [
			'enabled' => [
				'title' => __('Enable/Disable', 'holo-pay'),
				'type' => 'checkbox',
				'label' => __('Enable Holopay', 'holo-pay'),
				'default' => 'yes'
			],
			'title' => [
				'title' => __('Payment Method Title', 'holo-pay'),
				'type' => 'text',
				'description' => __('This is the title that users will see for this payment method during checkout.', 'holo-pay'),
				'default' => __('Pay with HoloPay', 'holo-pay'),
				'desc_tip' => true,
			],
			'description' => [
				'title' => __('Payment Method Description', 'holo-pay'),
				'type' => 'textarea',
				'description' => __('This is the description that users will see for this payment method during checkout.', 'holo-pay'),
				'default' => __('Pay securely using HoloPay. You will be redirected to HoloPay to complete your purchase.', 'holo-pay'),
				'desc_tip' => true,
			],
			'api_key' => [
				'title' => __('API Key', 'holo-pay'),
				'type' => 'text',
				'description' => __('Please enter your Holopay API Key.', 'holo-pay'),
				'default' => '',
			],
			'paypal_section' => [
				'title' => __('PayPal Settings', 'holo-pay'),
				'type' => 'title',
				'description' => __('Configure your PayPal payment settings below.', 'holo-pay'),
			],
			'paypal_client_id' => [
				'title' => __('PayPal Client ID', 'holo-pay'),
				'type' => 'text',
				'description' => __('Please enter your PayPal Client ID for PayPal payments.', 'holo-pay'),
				'default' => '',
			],
			'stripe_section' => [
				'title' => __('Stripe Settings', 'holo-pay'),
				'type' => 'title',
				'description' => __('Configure your Stripe payment settings below.', 'holo-pay'),
			],
			'stripe_publishable_key' => [
				'title' => __('Stripe Publishable Key', 'holo-pay'),
				'type' => 'text',
				'description' => __('Enter your Stripe Publishable Key.', 'holo-pay'),
				'default' => '',
			],
			'stripe_secret_key' => [
				'title' => __('Stripe Secret Key', 'holo-pay'),
				'type' => 'password',
				'description' => __('Enter your Stripe Secret Key.', 'holo-pay'),
				'default' => '',
			],
		];
	}

	public function generate_holopay_payment($order)
	{
		$products = [];
		foreach ($order->get_items() as $item) {
			$product = $item->get_product();
			$products[] = [
				'name' => $item->get_name(),
				'image' => wp_get_attachment_image_url($product->get_image_id(), 'full') ?: '',
				'quantity' => $item->get_quantity(),
				'price' => $item->get_total() / $item->get_quantity(),
			];
		}

		$params = [
			'orderId' => $order->get_id(),
			'currency' => $order->get_currency(),
			'return_url' => $this->get_return_url($order),
			'webhook' => add_query_arg('wc_id', $order->get_id(), $this->webhook_url),
			'email' => $order->get_billing_email(),
			'value' => $order->get_total(),
			'origin' => 'WOOCOMMERCE',
			'paypal_client_id' => $this->paypal_client_id,
			'products' => $products,
		];

		// Add Stripe keys if they exist
		if (!empty($this->stripe_publishable_key) && !empty($this->stripe_secret_key)) {
			$params['stripe_publishable_key'] = $this->stripe_publishable_key;
			$params['stripe_secret_key'] = $this->stripe_secret_key;
		}

		$route = "/v1/payments";
		$response = $this->holopay_post_authenticated_json_request($route, $params);

		if (is_wp_error($response)) {
			$errorMessage = __('Payment error', 'holo-pay'); 
			throw new \Exception(esc_html($errorMessage));
		} else if (isset($response['body']) && !empty($response['body'])) {
			$responseDecode = json_decode($response['body'], true);
			if (isset($responseDecode['error']) && !empty($responseDecode['error'])) {
				$errorMessage = __('Payment Gateway Error: ', 'holo-pay') . $responseDecode['status'].'-'.$responseDecode['error']; 
				throw new \Exception(esc_html($errorMessage));
			}

			// Store the unique ID from the payment response
			if (isset($responseDecode['data']['uniqid'])) {
				update_post_meta($order->get_id(), '_holopay_uniqid', $responseDecode['data']['uniqid']);
			}

			return isset($responseDecode['data']['url']) ? $responseDecode['data']['url'] : '';
		} else {
			$errorMessage = __('Payment Gateway Error: Empty response received.', 'holo-pay');
			throw new \Exception(esc_html($errorMessage));
		}
	}
	
	/**
	 * Process the payment and return the result
	 */
	public function process_payment($order_id)
	{
		$order = wc_get_order($order_id);
		try {
			$payment = $this->generate_holopay_payment($order);
			
			if ($payment) {
				return [
					'result' => 'success',
					'redirect' => $payment
				];
			} else {
				$errorMessage = __('Payment Gateway Error: Empty response received.', 'holo-pay');
				throw new \Exception(esc_html($errorMessage));
			}
		} catch (\Exception $e) {
            $message = $e->getMessage();
            WC()->session->set('refresh_totals', true);
            wc_add_notice($message, $notice_type = 'error');
            return array(
                'result' => 'failure',
                'redirect' => wc_get_checkout_url(),
				'message' => $message,
            );
        }
	}
	
	/**
	 * Handle webhooks
	 */
	public function webhook_handler()
	{
		global $woocommerce;
		
		$data = json_decode(file_get_contents('php://input'), true);
		
		$viWcID = 0;
		if (isset($_REQUEST['wc_id'])) {
			$viWcID = sanitize_text_field(wp_unslash($_REQUEST['wc_id']));
		}

		if (!$viWcID) {
			wp_send_json_error(['message' => 'Missing wc_id parameter'], 400);
			return;
		}

		$order = wc_get_order($viWcID);
		if (!$order) {
			wp_send_json_error(['message' => 'Order not found'], 404);
			return;
		}

		// Get the stored unique ID
		$stored_uniqid = get_post_meta($order->get_id(), '_holopay_uniqid', true);
		if (!$stored_uniqid) {
			wp_send_json_error(['message' => 'HoloPay ID not found for order'], 404);
			return;
		}

		// Validate the order status with the server
		$holopay_order = $this->valid_holopay_order($stored_uniqid);
		if (!$holopay_order) {
			wp_send_json_error(['message' => 'Failed to validate order status'], 500);
			return;
		}

		if ($holopay_order['status'] == 'COMPLETED') {
			$this->complete_order($viWcID);
			$order->payment_complete();
			wp_send_json_success(['message' => 'Order marked as completed and paid'], 200);
		} elseif ($holopay_order['status'] == 'WAITING') {
			$order->update_status('on-hold', sprintf(__('Awaiting payment confirmation', 'holo-pay')));
			wp_send_json_success(['message' => 'Order marked as on-hold'], 200);
		} else {
			wp_send_json_error(['message' => 'Unknown order status: ' . $holopay_order['status']], 400);
		}
	}
				
	public function process_admin_options() 
	{
		if (isset($_POST['woocommerce_holopay_api_key'])) {
			$api_key = sanitize_text_field($_POST['woocommerce_holopay_api_key']);
			
			// Test API key by making a simple request
			$test_response = $this->holopay_post_authenticated_json_request('/v1/test', [
				'api_key' => $api_key
			], '', 'POST');
			
			if (is_wp_error($test_response)) {
				WC_Admin_Settings::add_error(__('Invalid API Key. Please check your API key and try again.', 'holo-pay'));
				return false;
			} elseif (isset($test_response['response']['code']) && $test_response['response']['code'] !== 200) {
				WC_Admin_Settings::add_error(__('Invalid API Key. Please check your API key and try again.', 'holo-pay'));
				return false;
			}
		}
		
		return parent::process_admin_options();
	}

	function holopay_post_authenticated_json_request($route, $body = false, $extra_headers = false, $method="POST", $override_api_key = null)
	{
		
		$server = 'https://api.holopay.pro'; // Api Url
		$url = $server . $route;
		
		$uaString = 'Holopay WooCommerce (PHP ' . PHP_VERSION . ')';
		$apiKey = $override_api_key !== null ? $override_api_key : $this->api_key;
		$headers = array(
			'Content-Type'  => 'application/json',
			'User-Agent' => $uaString,
			'Authorization' => 'Bearer ' . $apiKey,
		);
	   
		if($extra_headers && is_array($extra_headers)) {
			$headers = array_merge($headers, $extra_headers);
		}
		
		$options = array(
			'method'  => $method,
			'timeout' => 10,
			'headers' => $headers,
		);

		if ($body !== false) {
			if (is_array($body)) {
				$options['body'] = json_encode($body);
			} else {
				$options['body'] = $body;
			}
		}
		
		return wp_remote_request($url, $options);
	}

	public function valid_holopay_order($order_uniqid)
	{
		$route = "/v1/orders/" . $order_uniqid;
		$response = $this->holopay_post_authenticated_json_request($route, false, false, 'GET');
		
		if (!is_wp_error($response) && isset($response['body'])) {
			$responseDecode = json_decode($response['body'], true);
			if (isset($responseDecode['data']) && isset($responseDecode['data']['order'])) {
				return $responseDecode['data']['order'];
			}
		}
		return false;
	}
	
	public function complete_order($wc_id) {
		global $woocommerce;
		$order = wc_get_order($wc_id);
		$order->update_status('completed');
	}

	public function option_exists($option_name) 
	{
		$value = get_option($option_name);
		return $value;
	}
}